@@ -52,6 +52,14 @@ def test_exists(path_rgb_byte_tif):
     assert not rasterio.shutil.exists('trash')
 
 
+def test_copy_fail_same_dataset(data):
+    """A dataset can't be copied to itself."""
+    path = str(data.join('RGB.byte.tif'))
+    with pytest.raises(RasterioIOError):
+        with rasterio.open(path) as src:
+            rasterio.shutil.copy(src, path, **src.profile)
+
+
 @pytest.mark.parametrize("pass_handle", (True, False))
 def test_copy(tmpdir, path_rgb_byte_tif, pass_handle):
 
@@ -143,6 +151,20 @@ def test_copyfiles(data, tmpdir):
         assert all(map(os.path.exists, actual.files))
 
 
+def test_copyfiles_same_dataset(data):
+    """A dataset can't be copied to itself."""
+    path = str(data.join('RGB.byte.tif'))
+    with pytest.raises(RasterioIOError):
+        rasterio.shutil.copyfiles(path, path)
+
+
+def test_copyfiles_same_dataset_another_name(data):
+    """A dataset can't be copied to itself by another name."""
+    path = str(data.join('RGB.byte.tif'))
+    with pytest.raises(RasterioIOError):
+        rasterio.shutil.copyfiles(path, 'file://' + path)
+
+
 def test_copyfiles_fail():
     with pytest.raises(RasterioIOError):
         rasterio.shutil.copyfiles('trash', 'whatever')
